from logging import getLogger
from re import findall
from typing import Optional

from chatushka.core.matchers.base import MatcherBase
from chatushka.core.models import MatchedToken, RegexMatchKwargs
from chatushka.core.transports.models import Message

logger = getLogger(__name__)


class RegexMatcher(MatcherBase):

    suffix = "regex"

    # def _cast_token(
    #     self,
    #     token: str,
    # ) -> Union[Hashable, Iterable[Hashable]]:
    #     return compile(token)

    async def _check(
        self,
        token: str,  # type: ignore
        message: Message,
    ) -> Optional[MatchedToken]:
        if founded := findall(token, message.text):
            kwargs = RegexMatchKwargs(matched=tuple(founded))
            return MatchedToken(
                token=token,
                kwargs=kwargs,
            )



from asyncio import sleep

from chatushka.bot.settings import get_settings
from chatushka.core.matchers import CommandsMatcher
from chatushka.core.transports.models import Message
from chatushka.core.transports.telegram_bot_api import TelegramBotApi

settings = get_settings()
pin_matcher = CommandsMatcher(
    prefixes=settings.command_prefixes,
    postfixes=settings.command_postfixes,
)


@pin_matcher("pin")
async def pin_handler(
    api: TelegramBotApi,
    message: Message,
    args: list[str],
) -> None:
    if not message.reply_to_message:
        await api.send_message(
            chat_id=message.chat.id,
            text="Для закрепа необходимо написать команду реплаем",
            reply_to_message_id=message.message_id,
        )

    try:
        pin_hours = int(args[0])
    except (ValueError, IndexError):
        pin_hours = None

    _ = await api.pin_chat_message(
        chat_id=message.chat.id,
        message_id=message.reply_to_message.message_id,
    )
    if pin_hours:
        await api.send_message(
            chat_id=message.chat.id,
            text=f"Через {pin_hours} ч. закреп будет убран",
            reply_to_message_id=message.message_id,
        )
        await sleep(pin_hours * 60)
        await api.unpin_chat_message(
            chat_id=message.chat.id,
            message_id=message.reply_to_message.message_id,
        )


@pin_matcher("unpin")
async def unpin_handler(
    api: TelegramBotApi,
    message: Message,
) -> None:
    await api.unpin_chat_message(
        chat_id=message.chat.id,
        message_id=message.reply_to_message.message_id,
    )



import signal
from asyncio import ensure_future, get_event_loop, sleep
from functools import partial
from logging import getLogger
from typing import Optional

from chatushka.__version__ import __URL__, __VERSION__
from chatushka.core.matchers import CommandsMatcher, EventsMatcher, EventTypes
from chatushka.core.models import MatchedToken
from chatushka.core.transports.models import Message
from chatushka.core.transports.telegram_bot_api import TelegramBotApi
from chatushka.core.transports.utils import check_preconditions

logger = getLogger(__name__)


async def _message_handler(
    bot_instance: "ChatushkaBot",
    message: Message,
    api: TelegramBotApi,
) -> None:
    await api.send_message(
        chat_id=message.chat.id,
        text=bot_instance.help_message_text,
        reply_to_message_id=message.message_id,
        parse_mode="markdown",
    )


class ChatushkaBot(EventsMatcher):
    def __init__(
        self,
        token: str,
        title: str = None,
        debug: bool = False,
    ) -> None:
        super().__init__()

        self.title = title or self.__class__.__name__
        self.debug = debug
        self.api = TelegramBotApi(token)
        self.add_handler(EventTypes.STARTUP, check_preconditions, include_in_help=False)

        bot_commands_matcher = CommandsMatcher(prefixes=("!", "/"))
        bot_commands_matcher.add_handler(
            ("start", "help"),
            partial(_message_handler, self),
            help_message="This message!",
        )
        self.add_matcher(bot_commands_matcher)

    @property
    def help_message_text(self):
        output = f"*CHATUSHKA BOT {__VERSION__}*\n_powered by ${__URL__}_"
        for help_message in self.help_messages:
            output += f"\n\n*{', '.join(help_message.tokens)}*\n> {help_message.message}"
        return output

    # pylint: disable=too-many-nested-blocks
    async def _loop(self) -> None:
        offset: Optional[int] = None
        while True:
            try:
                updates, latest_update_id = await self.api.get_updates(timeout=60, offset=offset)
            except Exception as err:  # noqa, pylint: disable=broad-except
                logger.error(err)
                await sleep(1)
                continue
            if updates:
                for update in updates:
                    message = update.message
                    logger.debug(
                        f'[ {message.chat.id} "{message.chat.title}" ] '
                        f'[ {message.user.id} "{message.user.readable_name}" ] -> '
                        f"{message.text}"
                    )
                    try:
                        for matcher in self.matchers:
                            matched_handlers = await matcher.match(self.api, message, should_call_matched=True)
                            for handler in matched_handlers:  # type: MatchedToken
                                logger.debug(
                                    f'[ {message.chat.id} "{message.chat.title}" ] '
                                    f'[ {message.user.id} "{message.user.readable_name}" ] <- '
                                    f"{handler.token}"
                                )
                    except Exception as err:  # noqa, pylint: disable=broad-except
                        logger.error(err)
                offset = latest_update_id + 1
            await sleep(1)

    async def _close(self) -> None:
        await self.call(self.api, EventTypes.SHUTDOWN)
        for matcher in self.matchers:
            if isinstance(matcher, EventsMatcher):
                await matcher.call(api=self.api, token=EventTypes.SHUTDOWN)

    async def serve(self) -> None:
        await self.call(self.api, EventTypes.STARTUP)
        loop = get_event_loop()
        for sig in (signal.SIGINT, signal.SIGTERM):
            try:
                loop.add_signal_handler(sig, callback=lambda: ensure_future(self._close()))
            except NotImplementedError:
                break
        for matcher in self.matchers:
            await matcher.init()
            if isinstance(matcher, EventsMatcher):
                await matcher.call(api=self.api, token=EventTypes.STARTUP)
        await self._loop()



from datetime import datetime, timedelta, timezone
from random import randrange

from chatushka.bot.settings import get_settings
from chatushka.core.matchers import CommandsMatcher
from chatushka.core.transports.models import ChatPermissions, Message
from chatushka.core.transports.telegram_bot_api import TelegramBotApi

settings = get_settings()
suicide_matcher = CommandsMatcher(
    prefixes=settings.command_prefixes,
    postfixes=settings.command_postfixes,
)


@suicide_matcher("suicide", "wtf")
async def suicide_handler(
    api: TelegramBotApi,
    message: Message,
) -> None:
    restrict_time = timedelta(minutes=randrange(1, 4 * 60))
    try:
        is_success = await api.restrict_chat_member(
            chat_id=message.chat.id,
            user_id=message.user.id,
            permissions=ChatPermissions(
                can_send_messages=False,
                can_send_media_messages=False,
                can_send_polls=False,
                can_send_other_messages=False,
            ),
            until_date=datetime.now(tz=timezone.utc) + restrict_time,
        )
    except ValueError:
        is_success = False
    if is_success:
        await api.send_message(
            chat_id=message.chat.id,
            text=f"Пользователь {message.user.readable_name} самовыпилился на {restrict_time}",
        )
        return None
    await api.send_message(
        chat_id=message.chat.id,
        text=f"Лапки коротковаты чтоб убить {message.user.readable_name}",
        reply_to_message_id=message.message_id,
    )



from chatushka.bot.matchers.admin import admin_matcher
from chatushka.bot.matchers.bobuk_jokes import jokes_matcher
from chatushka.bot.matchers.eight_ball import eight_ball_matcher
from chatushka.bot.matchers.helpers import helpers_matcher
from chatushka.bot.matchers.lukashenko import lukashenko_matcher
from chatushka.bot.matchers.suicide import suicide_matcher

__all__ = (
    "admin_matcher",
    "jokes_matcher",
    "eight_ball_matcher",
    "helpers_matcher",
    "suicide_matcher",
    "lukashenko_matcher",
)



from random import choice, randrange

from chatushka.bot.internal.data_dir import read_yaml_from_data_dir
from chatushka.bot.settings import get_settings
from chatushka.core.matchers import CommandsMatcher, RegexMatcher
from chatushka.core.transports.models import Message
from chatushka.core.transports.telegram_bot_api import TelegramBotApi

HELP_MESSAGE = (
    'Just think of a question that can be answered "Yes" or "No", concentrate very, very hard, and type command!'
)

settings = get_settings()
eight_ball_matcher = CommandsMatcher(
    prefixes=settings.command_prefixes,
    postfixes=settings.command_postfixes,
)
question_matcher = RegexMatcher()
eight_ball_matcher.add_matcher(question_matcher)


@eight_ball_matcher("8ball", "ball8", "b8", "8b", help_message=HELP_MESSAGE)
async def eight_ball_handler(
    api: TelegramBotApi,
    message: Message,
) -> None:
    answers = read_yaml_from_data_dir("eight_ball")
    await api.send_message(
        chat_id=message.chat.id,
        text=choice(answers["ru"]),
        reply_to_message_id=message.message_id,
    )


@question_matcher(r"\?", include_in_help=False)
async def eight_ball_answer_handler(
    api: TelegramBotApi,
    message: Message,
    matched: list[str],  # noqa, pylint: disable=unused-argument
) -> None:
    answers = read_yaml_from_data_dir("eight_ball")
    rand_int = randrange(8)
    if rand_int == 1:
        await api.send_message(
            chat_id=message.chat.id,
            text=choice(answers["ru"]),
            reply_to_message_id=message.message_id,
        )



from asyncio import run
from logging import DEBUG, INFO, WARNING, basicConfig, getLogger

from click import command, option

from chatushka import ChatushkaBot
from chatushka.bot.matchers import (
    admin_matcher,
    eight_ball_matcher,
    helpers_matcher,
    jokes_matcher,
    lukashenko_matcher,
    suicide_matcher,
)
from chatushka.bot.settings import get_settings

logger = getLogger()
settings = get_settings()


def make_bot(
    token: str,
    debug: bool,
) -> ChatushkaBot:
    instance = ChatushkaBot(token=token, debug=debug)
    instance.add_matcher(
        admin_matcher,
        jokes_matcher,
        eight_ball_matcher,
        helpers_matcher,
        suicide_matcher,
        lukashenko_matcher,
    )
    return instance


@command()
@option(
    "--token",
    "-t",
    required=True,
)
@option(
    "--debug/--no-debug",
    is_flag=True,
)
def cli_main(
    token: str,
    debug: bool,
) -> None:
    basicConfig(level=DEBUG if debug else INFO)
    getLogger("httpx").setLevel(WARNING)
    logger.debug("Debug mode is on".upper())
    bot = make_bot(token, debug)
    run(bot.serve())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['chatushka',
 'chatushka.bot',
 'chatushka.bot.internal',
 'chatushka.bot.matchers',
 'chatushka.bot.matchers.admin',
 'chatushka.bot.models',
 'chatushka.core',
 'chatushka.core.matchers',
 'chatushka.core.services',
 'chatushka.core.services.mongodb',
 'chatushka.core.transports',
 'chatushka.webui',
 'chatushka.webui.routes']

package_data = \
{'': ['*'], 'chatushka.bot': ['data/*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'aiocron>=1.6,<2.0',
 'click>=8.0.1,<9.0.0',
 'httpx>=0.22.0,<0.23.0',
 'motor>=2.5.0,<3.0.0',
 'pydantic[dotenv]>=1.8.2,<2.0.0']

setup_kwargs = {
    'name': 'chatushka',
    'version': '0.8.0',
    'description': 'Bot that can make your chat explode!',
    'long_description': '# Chatushka bot\n\nБот для чатиков (пока только для телеграмушки)\n\n\n# Usage\n\n## Installation\n\n```shell\npip install chatushka\n```\n\n## Start bot\n\n```shell\npython -m chatushka --token <telegrambotapitoken>\n```\n\n## Test bot\n\n- [x] добавить ботика в чат\n- [x] делегировать боту админские права\n- [x] написать в чатик /help\n\n```shell\npython -m chatushka --token <telegrambotapitoken>\n```\n',
    'author': 'Aleksandr Shpak',
    'author_email': 'shpaker@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/shpaker/chatushka',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from chatushka.core.transports.telegram_bot_api import TelegramBotApi


async def check_preconditions(
    api: TelegramBotApi,
) -> None:
    response = await api.get_me()
    if not response.can_join_groups:
        raise RuntimeError("Talk to @botfather and enable groups access for bot.")
    if not response.can_read_all_group_messages:
        raise RuntimeError("Talk to @botfather and disable the privacy mode.")
